#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "../include/oberon.h"

static oberon_context_t * ctx;
static oberon_module_t * mod;
static const char * module;
static const char * out_path;

static const char *
import_module(const char * name)
{
	assert(name);

	FILE * fp;
	char fname[256];
	snprintf(fname, 256, "%s.obn", name);
	fp = fopen(fname, "r");
	if(fp == NULL)
	{
		printf("can't open file %s\n", fname);
		exit(1);
		return NULL;
	}

	char * source;
	size_t len;

	fseek(fp, 0, SEEK_END);
	len = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	source = calloc(1, len + 1);
	fread(source, len, 1, fp);

	return source;
}

static void
init(int argc, char ** argv)
{
	int i = 1;
	out_path = ".";

	if(argc > 2 && strcmp(argv[i], "-d") == 0)
	{
		out_path = argv[i + 1];
		argc -= 2;
		i += 2;
	}

	if(argc != 2 || argv[i][0] == '-')
	{
		printf("usage: obn [-d out_dir] module\n");
		exit(1);
	}

	module = argv[i];
}

int
main(int argc, char ** argv)
{
	const char * code;

	init(argc, argv);

	code = import_module(module);

	ctx = oberon_create_context(import_module);
	oberon_set_out_directory(ctx, out_path);

	mod = oberon_compile_module(ctx, code);

	oberon_destroy_context(ctx);

	return 0;
}
